<?php

namespace Drupal\single_page_protection\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a configuration form for single page protection settings.
 */
class SinglePageProtectionConfigForm extends ConfigFormBase {

  /**
   * The database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new SinglePageProtectionConfigForm object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
        Connection $database,
        MessengerInterface $messenger
    ) {
    $this->database = $database;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new ProtectedPagePasswordForm(
          $container->get('database'),
          $container->get('messenger')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'single_page_protection_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['single_page_protection.settings'];
  }

  /**
   * Builds the configuration form for single page protection settings.
   *
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('single_page_protection.settings');

    $form['description'] = [
      '#markup' =>
      $this->t('Configure settings for single page protection.'),
    ];

    $form['protected_pages'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('URL'),
        $this->t('Password'),
        $this->t('Actions'),
      ],
      '#empty' => $this->t('No protected pages yet.'),
    ];

    $protected_pages = $this->loadProtectedPages();
    foreach ($protected_pages as $pid => $page) {
      $form['protected_pages'][$pid]['url'] = [
        '#type' => 'textfield',
        '#default_value' => $page['url'],
        '#disabled' => TRUE,
      ];
      $form['protected_pages'][$pid]['password'] = [
        '#type' => 'textfield',
        '#default_value' => $page['password'],
        '#disabled' => TRUE,
      ];
      $form['protected_pages'][$pid]['actions'] = [
        '#type' => 'submit',
        '#value' => $this->t('Delete'),
        '#submit' => ['::deleteProtectedPage'],
        '#name' => 'delete-' . $pid,
      ];
    }

    $form['new_protected_page'] = [
      '#type' => 'details',
      '#title' => $this->t('Add new protected page'),
      '#open' => TRUE,
    ];

    $form['new_protected_page']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
    ];

    $form['new_protected_page']['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Password'),
    ];

    $form['new_protected_page']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add'),
      '#submit' => ['::addProtectedPage'],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Loads protected pages from the database.
   *
   * @return array
   *   An associative array of protected pages.
   */
  private function loadProtectedPages() {
    $query = $this->database->select('single_page_protection', 'n')
      ->fields('n', ['pid', 'url', 'password'])
      ->execute();

    $results = [];
    foreach ($query as $record) {
      $results[$record->pid] = [
        'url' => $record->url,
        'password' => $record->password,
      ];
    }
    return $results;
  }

  /**
   * Deletes a protected page.
   *
   * {@inheritdoc}
   */
  public function deleteProtectedPage(array &$form, FormStateInterface $form_state) {
    $pid = str_replace('delete-', '',
          $form_state->getTriggeringElement()['#name']);
    $this->database->delete('single_page_protection')
      ->condition('pid', $pid)
      ->execute();
    $this->messenger->addMessage($this->t('Protected page deleted.'));
    $form_state->setRebuild(TRUE);
  }

  /**
   * Adds a new protected page.
   *
   * {@inheritdoc}
   */
  public function addProtectedPage(array &$form, FormStateInterface $form_state) {
    $url = trim($form_state->getValue(['new_protected_page', 'url']));
    $password = trim($form_state->getValue(['new_protected_page',
      'password',
    ]));

    if (!empty($url) && !empty($password)) {
      $this->database->insert('single_page_protection')
        ->fields([
          'url' => $url,
          'password' => $password,
        ])
        ->execute();
      $this->messenger->addMessage($this->t('New protected page added.'));
    }
    else {
      $this->messenger->addError(
            $this->t('URL and Password cannot be empty.'));
    }
    $form_state->setRebuild(TRUE);
  }

}
